package org.um.feri.spin_msc_generator;

import java.util.ArrayList;

import org.um.feri.spin_msc_generator.message.MSCMessage;
import org.um.feri.spin_msc_generator.process.MSCProcess;
import org.um.feri.spin_msc_generator.process.MSCVirtualProcess;

public class PrepareDocuments {
	
	private ArrayList<MSCDocument>	documentList = new ArrayList();
	private ArrayList<MSCMessage>	processMessageList = new ArrayList();
 	
	private String					comment = "";
	
	/**
	 * 
	 * Constructor for preparing documents and virtual documents
	 * 
	 * newProcessList			= list of processes in main MSC
	 * rewritenMessages			= list of messages in main MSC
	 * processList				= list of all processes in file
	 * virtualList				= list of virtual processes entered by user
	 * messageList				= list of all messages in file
	 * comment					= comment for main MSC
	 * 
	 * */
	
	public PrepareDocuments(ArrayList<MSCProcess> processList,	ArrayList<MSCMessage> messageList,
			String comment){
		
		// Without virtual processes
		
		this.comment 					= comment;
		
		createMainDocument(processList, messageList);
	}
	
	
	public PrepareDocuments(ArrayList<MSCProcess> newProcessList,	ArrayList<MSCMessage> rewritenMessages,
							ArrayList<MSCProcess> processList,		ArrayList<MSCVirtualProcess> virtualList,
							ArrayList<MSCMessage> messageList,		String comment){
		
		this.comment 					= comment;
		
		createMainDocument(newProcessList, rewritenMessages);
		createVirtualDocument(processList, virtualList, messageList);
	}
	
	private void createMainDocument(ArrayList<MSCProcess> newProcessList, ArrayList<MSCMessage> rewritenMessages){
		String name = "main";
		
		MSCDocument document = new MSCDocument(newProcessList, rewritenMessages, name, comment);
		documentList.add(document);
	}
		
	private void createVirtualDocument(ArrayList<MSCProcess> processList, ArrayList<MSCVirtualProcess> virtualList,
									   ArrayList<MSCMessage> messageList){
		
		MSCVirtualProcess	virtualProcess;
		MSCProcess			tmpProcess;
		String				tmpID = "";
		String				newID = "";
		String				actualID = "";
		
		ArrayList<MSCProcess>	tmpProcessList = new ArrayList();
		ArrayList<MSCMessage>	tmpMessageList = new ArrayList();
		MSCDocument				tmpMSCDocument;
		String					virtualProcessname = "";
		
		
		for (int i=0; i<virtualList.size(); i++) {
			// Sprehod skozi listo vseh virtualnih procesov			
			
			virtualProcess	= virtualList.get(i);
			actualID		= virtualProcess.getActualID();
			newID			= virtualProcess.getNewID();
			
			tmpProcess		= getProcess(processList, actualID);
			
			if (tmpID.equals(newID)){
				// Če je trenutni ID navideznega procesa enak prejšnjemu IDju
				
				tmpProcessList.add(tmpProcess);
				addMessageListForProcess(messageList, actualID);
			}
			else {
				// Če trenutni ID navideznega procesa ni enak prejšnjemu IDju
				
				if (tmpProcessList.size()!= 0){
					// Če trenutna lista procesov ni prazna jo dodaj v nov dokument
					
					addMessageListForProcess(messageList, actualID);
					tmpMSCDocument = new MSCDocument(tmpProcessList, processMessageList, virtualProcessname, "");
					processMessageList = new ArrayList();
					documentList.add(tmpMSCDocument);
				}
				tmpProcessList = new ArrayList();
				tmpProcessList.add(tmpProcess);
				addMessageListForProcess(messageList, actualID);
			}			
			tmpID		= newID;
			
			if (i == virtualList.size()-1){
				addMessageListForProcess(messageList, actualID);
				tmpMSCDocument = new MSCDocument(tmpProcessList, processMessageList, virtualProcessname, "");
				processMessageList = new ArrayList();
				documentList.add(tmpMSCDocument);
			}
			
			virtualProcessname = virtualProcess.getNewName();
		}
	}
	

	private MSCProcess getProcess(ArrayList<MSCProcess> processList, String actualID){
		MSCProcess tmpProcess;
		for (int i=0; i<processList.size(); i++){
			tmpProcess = processList.get(i);
			
			if (tmpProcess.getProcID().equals(actualID)){
				return tmpProcess;
			}
		}
		return null;
	}
	
	private void addMessageListForProcess(ArrayList<MSCMessage> messageList, String actualID){
		ArrayList<MSCMessage> tmpMessageList = new ArrayList();
		MSCMessage tmpMessage;
		
		for (int i=0; i<messageList.size(); i++){
			tmpMessage = messageList.get(i);
			
			if (tmpMessage.getFrom().equals(actualID) || tmpMessage.getTo().equals(actualID)){
				if (checkIfMessageAlreadyInList(processMessageList, tmpMessage)){
					// MESSAGE ALREADY IN LIST
					// DO NOTHING
				}
				else {
					// MESSAGE NOT IN THE LIST
					// ADD MESSAGE
					processMessageList.add(tmpMessage);
				}
			}
		}
	}

	private boolean checkIfMessageAlreadyInList(ArrayList<MSCMessage> messageList, MSCMessage tmpMessage){
		MSCMessage message;
		
		for (int i=0; i<messageList.size(); i++){
			message = messageList.get(i);
			
			if ((tmpMessage.getReceiveID() == message.getReceiveID()) && (tmpMessage.getSendID() == message.getSendID())){
				return true;
			}
		}
		return false;
	}
	
	
	public ArrayList<MSCDocument> getDocumentList(){
		return documentList;
	}
}