package org.um.feri.spin_msc_generator;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.um.feri.spin_msc_generator.arguments.ArgumentsContainer;
import org.um.feri.spin_msc_generator.generator.MSCGenerator;
import org.um.feri.spin_msc_generator.message.MSCMessage;
import org.um.feri.spin_msc_generator.message.MSCMessageRewriter;
import org.um.feri.spin_msc_generator.message.MessageRenamer;
import org.um.feri.spin_msc_generator.message.RenamedMessage;
import org.um.feri.spin_msc_generator.parser.MSCParser;
import org.um.feri.spin_msc_generator.process.MSCProcess;
import org.um.feri.spin_msc_generator.process.MSCProcessJoiner;
import org.um.feri.spin_msc_generator.process.MSCVirtualProcess;
import org.um.feri.spin_msc_generator.ui.UserInterface;

public class St2msc {
	
	/**
	 * Class St2msc is the root of the st2msc hierarchy. It implements methods for 
	 * communication with user and initialization of all needed objects.
	 * 
	 *
	 * @version 2.0
	 * @since st2msc 2.0 
	 * @author UM - FERI, Laboratorij za mikroracunalniske sisteme,  Tim KOVSE
	 * 
	 * */
	
	public static	String st2mscVersion	= "2.0.0";
	public static	String st2mscDate		= "2009.05";
	
	ArgumentsContainer			argContainer;
	
	/** processList */
	ArrayList<MSCProcess>		processList;
	/** messageList */
	ArrayList<MSCMessage>		messageList;
	
	ArrayList<MSCMessage>		messageNotReceivedList;
	
	ArrayList<MSCDocument>		mscDocuments;
	
	ArrayList<RenamedMessage>	rnMessageList;
	
	UserInterface ui			= new UserInterface();
		
	
	/**
	 * Creates a new st2msc object and calls mainFunction.
	 * 
	 * @param _input_file is the input file variable, which contains the path to Spin trail.out file
	 * @param _output_file is the output file variable for generated SDL Z.106, Z.120 compliant MSC file
	 */
	
	public St2msc(ArgumentsContainer argContainer){
		this.argContainer = argContainer;
		mainFunction();
	}
	
	/**
	 * 
	 * Method mainFunction is the main method in st2msc class. It initializes a parser, with Spin trail
	 * out file as an argument. Parser returns objects of parser, which are list of all processes in file and
	 * list of all messages.
	 * 
	 * @param _input_file is the input file variable, which contains the path to Spin trail.out file
	 * @param _output_file is the output file variable for generated SDL 106, 120 compliant MSC file
	 *
	 * */
	
	private void mainFunction(){
		
		if (!(argContainer.getInput().compareTo("") == 0)) {
			ui.docParsing();
			MSCParser parser		= new MSCParser(argContainer.getInput());
			ui.docParsed();
	
			processList				= parser.getProcessList();
			messageList				= parser.getMessageList();
			messageNotReceivedList	= parser.getNotReceivedMessageList();
			
			if (argContainer.getShowProcessList()){
				ui.detectedItems(processList.size(), messageList.size(), messageNotReceivedList.size());
				if ((processList.size() == 0) && (messageList.size()==0)){
					ui.noItemsFound();
				}
				else {
					ui.showProcesses(processList);
				}
			}
			
			if (argContainer.getShowMessageList()){
				ui.showMessages(messageList);
			}
			
			if (argContainer.getShowUniqueMessageList()){
				ui.showUniqueMessages(messageList);
			}

			if (!(argContainer.getOutput().compareTo("") == 0))
				documentPreparation(processList, messageList);
		}
	}
	
	/**
	 * 
	 * Method documentPreparation is used for ...
	 *
	 * @param processList list of all detected processes
	 * @param messageList list of all detected messages
	 *
	 * */
	
	private void documentPreparation(ArrayList<MSCProcess> processList, ArrayList<MSCMessage> messageList){
		String userJoinedProcesses = "";
		ArrayList<MSCProcess> newProcessList;
		ArrayList<MSCVirtualProcess> virtualList = new ArrayList();
		ArrayList<MSCMessage> newMessageList = new ArrayList();
		
		String comment = "";
		String messageRename = "";
		boolean hide_message_parameters = true;
		

		
		
		/* Temporary solution!
		 * @TODO
		 */
		
		ArrayList<String> virtualPList = argContainer.getVirtualProcessListArguments();
		
		for (int i=0; i<argContainer.getVirtualProcessListArguments().size(); i++){
			userJoinedProcesses = virtualPList.get(i) + " ";
		}
		
		hide_message_parameters = argContainer.getHideMessageParameters();
		
		if (argContainer.getComment().length()>0){
			comment = argContainer.getComment();
		}
		

		if (argContainer.getVirtualProcessListArguments().size() != 0){
			MSCProcessJoiner mpj = new MSCProcessJoiner(processList, userJoinedProcesses);
			virtualList		= mpj.getVirtualProcesses();
			newProcessList	= mpj.getGeneratedProcessList(processList, virtualList);
		
			//ui.showProcesses(newProcessList);
			
			MSCMessageRewriter mmr = new MSCMessageRewriter(virtualList, messageList); 
			newMessageList = mmr.getRewrittenMessageList();
			
			
			if (argContainer.getRenameMessageListArguments().length()>0){
				
				/* Temporary solution!
				 * @TODO
				 */				
				
				for (int i=0; i<argContainer.getRenameMessageListArguments().length(); i++){
					messageRename = argContainer.getRenameMessageListArguments();
				}
				//System.out.println(messageRename);
								
				MessageRenamer msgRenamer = new MessageRenamer(messageRename);
				rnMessageList = msgRenamer.getRenamedMessage();
				
				PrepareDocuments pd = new PrepareDocuments(newProcessList, newMessageList, 
						processList, virtualList, messageList, comment);
				MSCGenerator  mscGen = new MSCGenerator(pd.getDocumentList(), rnMessageList,argContainer);
				
			}
			else {
				PrepareDocuments pd = new PrepareDocuments(newProcessList, newMessageList, 
						processList, virtualList, messageList, comment);
				MSCGenerator  mscGen = new MSCGenerator(pd.getDocumentList(), argContainer);
			}
		}
		
		else {
			
			//comment = ui.addComment();
			//hide_message_parameters = ui.showMessageParameters();
			
			
			if (argContainer.getRenameMessageListArguments().length()>0){
				
				for (int i=0; i<argContainer.getRenameMessageListArguments().length(); i++){
					messageRename = argContainer.getRenameMessageListArguments();
				}
				
				//System.out.println(messageRename);
								
				MessageRenamer msgRenamer = new MessageRenamer(messageRename);
				rnMessageList = msgRenamer.getRenamedMessage();
				
				PrepareDocuments pd = new PrepareDocuments(processList, messageList, comment);
				MSCGenerator  mscGen = new MSCGenerator(pd.getDocumentList(), rnMessageList, argContainer);
				
			}
			else {
				PrepareDocuments pd = new PrepareDocuments(processList, messageList, comment);
				MSCGenerator  mscGen = new MSCGenerator(pd.getDocumentList(), argContainer);
			}
		}
	}
}
