package org.um.feri.spin_msc_generator.arguments;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;


public class ArgumentParser {
	
	private ArrayList<Options> 	validOptions;
	private ArrayList<Options> 	userOptions;
	
	private ArrayList<String>	virtualProcessListArguments = new ArrayList();
	private String	renameMessageListArguments	= "";
	
	private ArgumentsContainer argContainer	= new ArgumentsContainer();
	
	String[] args;
	
	boolean	error		= false;
	String	errorReason = "";
		
	public ArgumentParser(){
		validOptions	= new ArrayList();
		userOptions		= new ArrayList(); 
		
		validOptions	= addSt2mscOptions();
	}
	
	public void parse(String[] args){
		this.args		= args;
		int	argIndex	= args.length-1;
		int argsErrorIndex	= areArgumentsOk(args);
		
		
		if (argsErrorIndex == -1){
			if (areDependenciesOk(args)){
				startParsing(args);
				
				if (virtualProcessListArguments.size() > 0){
					argContainer.setVirtualProcessListArguments(virtualProcessListArguments);
				}			
				if (renameMessageListArguments.length() > 0){
					argContainer.setRenameMessageListArguments(renameMessageListArguments);
				}
			}
			else {
				setError("Check dependencies.");
			}
		}
		else {
			setError("Argument " + args[argsErrorIndex] + " is not on the options list.");
		}
	}
	
	private void startParsing(String[] arguments){
		
		int	argIndex	= arguments.length-1;
		
		int parResult	= 0; // Parameter result: 0 = false, 0< = true & skipStep
		
		for (int i=0; i<arguments.length; i++){
			if (arguments[i].startsWith("-")){
				// IS SWITCH
				if (isOptionValid(arguments[i])){
					// OPTION IS VALID
					if (optionNeedsParameter(arguments[i])){
						// OPTION NEEDS PARAMETERS
						if (i<argIndex){
							// SOME PARAMETERS AFTER THIS EXIST
							
							parResult = getParameterValue(arguments[i], arguments, i);
							
							if (-1<parResult){
							// PARAMETER IS OK
								
								i = i + parResult;
							}
							else {
								//PARAMETER NOT VALID
							}
						}
						else {
							setError("Argument " + arguments[i] + " needs parameters");
						}
					}
					else {
						// OPTION DOES NOT NEED PARAMETERS
						parResult = getParameterValue(arguments[i], arguments, i);
					}					
				}
				else {
					// OPTION IS NOT VALID
					setError(arguments[i] + " is not a valid argument");
					break;
				}
			}
			else {
				// IS DATA				
				setError("Argument " + arguments[i] + " is not a valid argument");
				break;
			}
		}
	}
	
	boolean isOptionValid(String argument){
		Options option;
		
		for (int i=0; i<validOptions.size(); i++){
			option = validOptions.get(i);
			if ((argument.equals(option.getOptionTypeShort()) || argument.equals(option.getOptionTypeLong()))){
				return true;
			}			
		}	
		return false;
	}	

	boolean optionNeedsParameter(String argument){
		Options option;
		
		for (int i=0; i<validOptions.size(); i++){
			option = validOptions.get(i);
			if ((argument.equals(option.getOptionTypeShort()) || argument.equals(option.getOptionTypeLong()))){
				return option.needData;
			}			
		}	
		return false;
	}

	
	private int getParameterValue(String argument, String[] arguments, int current){
		if (argument.equals("-i") || argument.equals("-input")){
			if (arguments[current+1].startsWith("-")){
				return -1;
			}
			else {
				File file = new File(arguments[current+1]);
				if (file.canRead()){
					argContainer.setInput(arguments[current+1]);
					return 1;
				}
				else {
					System.out.println("Cannot read input file");
					return -1;
				}
			}
		}
		
		if (argument.equals("-o") || argument.equals("-output")){
			if (arguments[current+1].startsWith("-")){
				return -1;
			}
			else {
				File file = new File(arguments[current+1]);
				if (file.exists()){
					file.delete();
					argContainer.setOutput(arguments[current+1]);
					return 1;
				}
				else {
					try{
						file.createNewFile();
						file.delete();
						argContainer.setOutput(arguments[current+1]);
						return 1;
					}
					catch (IOException ex){
						return -1;
					}
				}
			}
		}
		
			
		if (argument.equals("-vpl") || argument.equals("-virtualprocesslist")){
			boolean	passVirtualRequirements = false;
			int		skipStep	= 0;
			
			VirtProcessRegEx vpre = new VirtProcessRegEx();
			
			for (int i=current+1; i<arguments.length; i++){
				if (arguments[i].startsWith("-")){
					if (passVirtualRequirements){
						if (vpre.isRegExOk(arguments[i])){
							virtualProcessListArguments.add(arguments[i]);
							skipStep++;
						}
						else {
							return skipStep;						
						}
					}
					else {
						return -1;
					}
				}
				else {
					if (vpre.isRegExOk(arguments[i])){
						virtualProcessListArguments.add(arguments[i]);
						passVirtualRequirements = true;
						skipStep++;
					}
					else {
						return skipStep;
					}
				}
			}
			return skipStep;
		}
		
		if (argument.equals("-c") || argument.equals("-comment")){
			int		skipStep	= 0;
			boolean passCommentRequirements = false;
			String comment = "";
			
			for (int i=current+1; i<arguments.length; i++){
				if (arguments[i].startsWith("-")){
					if (passCommentRequirements){
						argContainer.setComment(comment);
						return skipStep;
					}
					else {
						return -1;
					}
				}
				else {
					passCommentRequirements = true;
					skipStep++;
					comment = comment + " " + arguments[i] + " ";
				}
			}
			argContainer.setComment(comment);
			return skipStep;
		}
		
		
		
		if (argument.equals("-rml") || argument.equals("-renamemessagelist")){
			boolean	passRenameRequirements = false;
			int		skipStep	= 0;
			
			RnMessageRegEx	rmre = new RnMessageRegEx();
			
			for (int i=current+1; i<arguments.length; i++){
				if (arguments[i].startsWith("-")){
					if (passRenameRequirements){
						if (rmre.isRegExOk(arguments[i])){
							if (renameMessageListArguments.length() == 0){
								renameMessageListArguments = arguments[i];
							}
							else {
								renameMessageListArguments = renameMessageListArguments + " " + arguments[i];
							}
							skipStep++;
						}
						else {
							return skipStep;						
						}
					}
					else {
						return -1;
					}
				}
				else {
					if (rmre.isRegExOk(arguments[i])){
						if (renameMessageListArguments.length() == 0){
							renameMessageListArguments = arguments[i];
						}
						else {
							renameMessageListArguments = renameMessageListArguments + " " + arguments[i];
						}
						passRenameRequirements = true;
						skipStep++;
					}
					else {
						return skipStep;
					}
				}
			}
				
			
			return skipStep;
		}
		
		if (argument.equals("-spl") || argument.equals("-showprocesslist")){
			argContainer.setShowProcessList(true);
		}
		
		if (argument.equals("-sml") || argument.equals("-showmessagelist")){
			argContainer.setShowMessageList(true);
		}		
		
		if (argument.equals("-suml") || argument.equals("-showuniquemessagelist")){
			argContainer.setShowUniqueMessageList(true);
		}	
		
		if (argument.equals("-h") || argument.equals("-help")){
			argContainer.setShowHelp(true);
		}
		
		if (argument.equals("-hmp") || argument.equals("hidemessageparameters")){
			argContainer.setHideMessageParameters(true);
		}

		return -1;
	}
	
	private int areArgumentsOk(String[] arguments){
		int isOK = -1;
				
		for (int i=0; i<arguments.length; i++){
			if (arguments[i].startsWith("-")){
				isOK = i;
				for (int j=0; j<validOptions.size(); j++){
					Options option;
					option = validOptions.get(j);
										
					if (arguments[i].equals(option.optionTypeShort) || arguments[i].equals(option.optionTypeLong)){
						isOK = -1;
						break;
					}
				}
				if (isOK != -1){
					return isOK;
				}
			}
		}
		return -1;
	}
	
	
	private boolean areDependenciesOk(String[] arguments){
		
		boolean dependenciesOK 	= false;
		boolean input			= false;
		boolean output			= false;
		boolean needInput		= false;
		boolean needOutput		= false;
		
		Options opt;
		
		for (int i=0; i<arguments.length; i++){
			for (int j=0; j<validOptions.size(); j++){
				opt = validOptions.get(j);
				if (arguments[i].equals(opt.optionTypeLong) || arguments[i].equals(opt.optionTypeShort)){
					if (opt.needInput == true)
						needInput = true;
					if (opt.needOutput == true)
						needOutput = true;
				}
			}
			if ((arguments[i].equals("-i")) || (arguments[i].equals("-input"))) {
				input = true;
			}
			if ((arguments[i].equals("-o")) || (arguments[i].equals("-output"))) {
				output = true;
			}
		}
		
		if (needInput){
			if (needOutput){
				if (input && output)
					dependenciesOK = true;
				else
					dependenciesOK = false;
			}
			else {
				if (input){
					dependenciesOK = true;
				}
				else {
					dependenciesOK = false;
				}
			}
		}
		else {
			dependenciesOK = true;
		}

		return dependenciesOK;
	}
		
	private void setError(String errorReason){
		this.error			= true;
		this.errorReason	= errorReason;
	}
	
	private ArrayList<Options> addSt2mscOptions(){
		ArrayList<Options> st2mscOptions = new ArrayList();
		st2mscOptions.add(new Options("-i",		"-input",					true, 	false,	false	));
		st2mscOptions.add(new Options("-o",		"-output",					true, 	true,	false	));
		st2mscOptions.add(new Options("-h",		"-help",					false,	false,	false	));
		st2mscOptions.add(new Options("-v",		"-verbose",					false,	false,	false	));		
		st2mscOptions.add(new Options("-vpl",	"-virtualprocesslist", 		true,	true,	true	));
		st2mscOptions.add(new Options("-rml", 	"-renamemessagelist",		true,	true,	true	));
		st2mscOptions.add(new Options("-spl",	"-showprocesslist",			false,	true,	false	));
		st2mscOptions.add(new Options("-sml",	"-showmessagelist",			false,	true,	false	));
		st2mscOptions.add(new Options("-suml",	"-showuniquemessagelist",	false,	true,	false	));
		st2mscOptions.add(new Options("-hmp",	"-hidemessageparameters",	false,	true,	true	));
		st2mscOptions.add(new Options("-c",		"-comment",					true,	true,	true	));
		return st2mscOptions;
	}
	
	public boolean isError(){
		return error;
	}
	public String getErrorReason(){
		return errorReason;
	}
	
	public ArgumentsContainer getArguments(){
		return argContainer;
	}
	
	private static class Options {
		
		private String	optionTypeShort;
		private String 	optionTypeLong;
		private boolean needData;
		
		private boolean needInput;
		private boolean needOutput;
		
		private Options(String optionTypeShort, String optionTypeLong, boolean needData, boolean needInput, boolean needOutput){
			this.optionTypeShort	= optionTypeShort;
			this.optionTypeLong		= optionTypeLong;
			this.needData			= needData;
			this.needInput			= needInput;
			this.needOutput			= needOutput;
		}

		private String getOptionTypeShort(){
			return this.optionTypeShort;
		}
		private String getOptionTypeLong(){
			return this.optionTypeLong;
		}
		private boolean getNeedData(){
			return this.needData;
		}
		private boolean getNeedInput(){
			return this.needInput;
		}
		private boolean getNeedOutput(){
			return this.needOutput;
		}
	}
}
