package org.um.feri.spin_msc_generator.generator;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;

import org.um.feri.spin_msc_generator.MSCDocument;
import org.um.feri.spin_msc_generator.MSCWriter;
import org.um.feri.spin_msc_generator.arguments.ArgumentsContainer;
import org.um.feri.spin_msc_generator.message.MSCMessage;
import org.um.feri.spin_msc_generator.message.RenamedMessage;
import org.um.feri.spin_msc_generator.msc_parameters.MSCCommentParameters;
import org.um.feri.spin_msc_generator.msc_parameters.MSCDocumentParameters;
import org.um.feri.spin_msc_generator.msc_parameters.MSCMessageParameters;
import org.um.feri.spin_msc_generator.msc_parameters.MSCProcessParameters;
import org.um.feri.spin_msc_generator.process.MSCProcess;
import org.um.feri.spin_msc_generator.ui.UserInterface;

//


public class MSCGenerator {

	/**
	 * Class MSCGenerator is the class with the task of creating MSC document. It implements methods for 
	 * adding MSC compliant constructs to output file.
	 * 
	 *
	 * @version 2.0
	 * @since MSCSpin 2.0 
	 * @author UM - FERI, Laboratorij za mikroracunalniske sisteme,  Tim KOVSE
	 * 
	 * */
		
	private final String	toolAuthor	= "UM FERI - LMS - Tim KOVSE";
	private final String	toolName	= "st2msc";
	private final String	toolVersion	= "st2msc 2.0";
	private final String	toolFormat	= "MSC92";
	private final String	virtualName	= "main";
	
	private int document_width;
	private int document_height;
	
	int width_between_processes = 100;
	
	int height_between_messages_factor	= 100;
	
	
	int comment_width	= 400;
	String comment		= "";
	
	public boolean show_parameters = true;

	private int offset_y;
	private int offset_x;
	
	private ArrayList<RenamedMessage> rnMessageList = new ArrayList();
	
	private ArgumentsContainer argContainer;
	
	private MSCWriter write;
	
	
		
	/**
	 * 
	 * MSCGenerator construct
	 * 
	 * @param _processList list of all detected processes in input file
	 * @param _messageList list of all detected messages in input file
	 * @param _comment comment on created MSC 
	 * @param _show_message_parameters is a variable which tells if message parameters should be shown in MSC file
	 * 
	 * */	
	
	public MSCGenerator(ArrayList<MSCDocument> documentList, ArgumentsContainer argContainer){
		this.argContainer = argContainer;
		createFramework(documentList);
	}
	
	public MSCGenerator(ArrayList<MSCDocument> documentList, ArrayList<RenamedMessage> rnMessageList, ArgumentsContainer argContainer){
		this.rnMessageList = rnMessageList;
		this.argContainer = argContainer; 
		
		createFramework(documentList);
		
	}
	
	public void createFramework(ArrayList<MSCDocument> documentList){
		write = new MSCWriter(argContainer.getOutput());
		
		MSCDocument document;
		
		if (documentList.size() == 1){
			createDocument(documentList.get(0));
		}
		else if (documentList.size()>1){
			MSCDocumentParameters documentParameters = new MSCDocumentParameters(1000, 2000, 1500, 890, toolName, toolVersion, toolAuthor, toolFormat, virtualName);
			write.write(documentParameters.getStartMSCDocument(true), false);
			
			for (int i=0; i<documentList.size(); i++){
				document = documentList.get(i);
				createDocument(document);
			}
			write.write(documentParameters.getEndMSCDocument(true), false);	
		}
		
		write.closeFile();
	}

	
	public void createDocument(MSCDocument document){
		MSCDocumentParameters	mscDocumentParam;
		MSCCommentParameters	mscCommentParam;
				
		String	name			= document.getName();

		ArrayList<MSCProcess>	processList = document.getProcessList();
		ArrayList<MSCMessage>	messageList = document.getMessageList();
		
		show_parameters 		= argContainer.getHideMessageParameters();
		comment					= document.getComment();
		
		document_width			= getDocumentWidth(processList, comment);
		document_height			= getDocumentHeight(messageList);
		
		int page_width			= document_width + 400;
		int page_height			= document_height + 40;
		
		mscDocumentParam		= new MSCDocumentParameters(document_width, document_height, page_width, 
								page_height, toolName, toolVersion, toolAuthor, toolFormat, name);
		
		write.write(mscDocumentParam.getStartMSCDocument(false), false);
		
		if (document.getComment().length() != 0){
			mscCommentParam 		= new MSCCommentParameters(document_width,400, document.getComment()); 
			write.write(mscCommentParam.getComment(), false);
		}
		
		createProcess(processList, messageList, page_height);
	
		write.write(mscDocumentParam.getEndMSCDocument(false), false);
		
	}
	
	public void createProcess(ArrayList<MSCProcess> processList, ArrayList<MSCMessage> messageList, int page_height){
		// INSTANCE
		//
		offset_x				= width_between_processes;
		offset_y 				= 0;
		int proc_line_offset 	= 0;
		int head_middle 		= 0;
		int process_head_width	= 0;
		int head_width			= 0;
		
		String procName			= "";
		String procID			= "";
		
		MSCProcess				mscProcess;
		MSCMessage				mscMessage;
		MSCProcessParameters	mscProcessParam;
		ArrayList<MSCProcessParameters> processParametersList = new ArrayList();
				
		for (int i=0; i<processList.size(); i++){
			mscProcess 			= processList.get(i);
			procName			= mscProcess.getProcName();
			procID				= mscProcess.getProcID();
			proc_line_offset	= mscProcess.getProcessLineID();
			
			process_head_width = getProcessWidth(procName);
			
			
						
			if (!procName.equals(":init:")){
			
				// @TODO SETTING OFFSET FOR PROCESS CREATION
				if (proc_line_offset == 0){
					offset_y = 80;				
				}
				else {
					offset_y = 80; 
				}
			
				int procEND = page_height - 350; // -350 because of process head size and instance name
			
				head_middle = offset_y + (process_head_width/2);
			
			
				mscProcessParam = new MSCProcessParameters(procID, procName, offset_x, offset_y, head_middle, procEND);
				head_width = offset_x + (mscProcessParam.getHeadWidth()/2);
			
				processParametersList.add(mscProcessParam);
			
				offset_x = offset_x + process_head_width + width_between_processes;

			}
		}
		for (int j=0; j<processParametersList.size(); j++){
			
			mscProcessParam = processParametersList.get(j);
			procID			= mscProcessParam.getProcID();
			head_middle		= mscProcessParam.getHeadMiddle();
			
			write.write(mscProcessParam.getProcessStart(), false);
			write.write(getMessages(procID, head_middle, processParametersList, messageList, processList), false);
			write.write(mscProcessParam.getProcessEnd(), false);
		}
	}
	
	
	
	public int getDocumentWidth(ArrayList<MSCProcess> processList, String comment){
		String processName 			= "";
		int width 					= 40;	//Starting and ending width of processes
		int width_between_processes = 100;
		int comment_width			= 400;
		
		MSCProcess process 			= null;
		
		for (int i=0; i<processList.size(); i++){
			process 	= processList.get(i);
			processName = process.getProcName();
			width 		= width + getProcessWidth(processName) + width_between_processes;	
		}
		if (comment.length() != 0){
			width 		= width + comment_width;
		}
		return width;
	}
	
		
	private int getProcessWidth(String procName){
		int defaultSize		= 120;		// Default size without text
		int incrementSize 	= 12;		// Char size
		int width 			= defaultSize + (procName.length() * incrementSize);	// Calculate process width	

		return width;
	}
	
	public int getDocumentHeight(ArrayList<MSCMessage> messageList){
		int maxMessageID = 0;
		int height = 800;

		MSCMessage mscMessage;
		
		for (int i=0; i<messageList.size(); i++){
			mscMessage = messageList.get(i);
			
			if (mscMessage.getReceiveID()>maxMessageID){
				maxMessageID = mscMessage.getReceiveID();
			}
		}
		
		int numMessages = messageList.size();
		
		height = height + maxMessageID * height_between_messages_factor;
		
		return height;
	}
	
	private StringBuffer getMessages(String _procID, int _head_middle, ArrayList<MSCProcessParameters> _processParamList,
			ArrayList<MSCMessage> messageList, ArrayList<MSCProcess> processList){
		
		StringBuffer str = new StringBuffer();
		MSCMessage message = null;
		MSCProcessParameters processParam = null;
		MSCMessageParameters messageParam = null;
		
		int start_x	= 0;
		int end_x	= 0;
		
		int start_y = 0;
		int end_y	= 0;
		
		int yOffset = 200;
			
		boolean environment = false;
		
		String toFrom;
		String messageName;
		
		for (int i=0; i<messageList.size(); i++){
			message = messageList.get(i);
		
			if (message.getFrom().equals(_procID)){
		
				start_x = _head_middle;
				if (!message.getFrom().equals(message.getTo())){
					for (int j=0; j<_processParamList.size(); j++){
						processParam = _processParamList.get(j);
						if (message.getTo().equals(processParam.getProcID())){
							end_x = processParam.getHeadMiddle();
							environment = false;
							break;
						}
						else {
							environment = true;
							if (Integer.parseInt(_procID) > Integer.parseInt(message.getTo())){
								end_x = 0;
							}
							else {
								end_x = document_width;
							}
						}
					}
					
					toFrom = getProcessName(processList, message.getTo());
										
					if (rnMessageList.size() != 0){
						messageName = getRenamedMessage(message.getMessage());
					}
					else {
						messageName = message.getMessage();
					}
					
					messageParam = new MSCMessageParameters("OUT", "TO", messageName, message.getValue(), 
							start_x, message.getSendID()*height_between_messages_factor+yOffset, end_x, 
							message.getReceiveID()*height_between_messages_factor+yOffset, show_parameters, environment, toFrom);
					environment = false;
					str.append(messageParam.getCifMessage() + "\n");
					str.append(messageParam.getMessage() + "\n");
				}
			}
			if (message.getTo().equals(_procID)){
				end_x = _head_middle;
				if (!message.getFrom().equals(message.getTo())){
					for (int k=0; k<_processParamList.size(); k++){
						processParam = _processParamList.get(k);
						if (message.getFrom().equals(processParam.getProcID())){
							start_x = processParam.getHeadMiddle();
							environment = false;
							break;
						}
						else {
							environment = true;
							if (Integer.parseInt(_procID) > Integer.parseInt(message.getFrom())){
								start_x = 0;
							}
							else {
								start_x = document_width;
							}
						}
					}
					
					toFrom = getProcessName(processList, message.getFrom());
					
					if (rnMessageList.size() != 0){
						messageName = getRenamedMessage(message.getMessage());
					}
					else {
						messageName = message.getMessage();
					}
					
					messageParam = new MSCMessageParameters("IN", "FROM", messageName, message.getValue(), 
							start_x, message.getSendID()*height_between_messages_factor+yOffset, end_x, 
							message.getReceiveID()*height_between_messages_factor+yOffset, show_parameters, environment, toFrom);
				
					environment = false;
					str.append(messageParam.getCifMessage() + "\n");
					str.append(messageParam.getMessage() + "\n");
				}
			}
		}
		return str;		
	}
	
	
	private String getRenamedMessage(String messageName){
		
		RenamedMessage renamedMessage;
		
		for (int i=0; i<rnMessageList.size(); i++){
			renamedMessage = rnMessageList.get(i);
			
			if (renamedMessage.getActualName().equals(messageName)){
				return renamedMessage.getNewName();
			}
		}
		return messageName;
	}
	
	private String getProcessName(ArrayList<MSCProcess> processList, String _procID){
		String procName;
		MSCProcess process;
		
		for (int i=0; i<processList.size(); i++){
			process = processList.get(i);
			
			if (process.getProcID().equals(_procID)){
				return process.getProcName();
			}
		}
		return "";		
	}
	
}
