package org.um.feri.spin_msc_generator.message;

/**
 * 
 * MSCMessage class is used for creating new MSCMessage instance
 * 
 * @author Tim KOVSE - UM FERI
 * 
 * @version 1.0
 * 
 *    
 * */

public class MSCMessage {
	
	private String	message;
	private String	value;
	private String	from;			// Message from
	private String	to;				// Message to
	
	private int		sendID;			// ID of line, where the message was sent
	private int		receiveID;		// ID of line, where the message was received
	
	/**
	 * 
	 * MSCMessage constructor is used for storing temporary message structure.
	 * 
	 * @param _message name of message 
	 * @param _value value of message
	 * @param _from  process id that sent this message
	 * @param _sendID id of line in which this message was sent
	 * 
	 * */
	
	public MSCMessage(String _message, String _value, String _from, int _sendID){
		this.message	= _message;
		this.value		= _value;
		this.from		= _from;
		this.sendID		= _sendID; 
	}
	
	/**
	 * 
	 * MSCMessage constructor is used for storing message structure.
	 * 
	 * @param _message name of message 
	 * @param _value value of message
	 * @param _from  process id that sent this message
	 * @param _to process id that received this message
	 * @param _sendID id of line in which this message was sent
	 * @param _receiveID id of line in which this message was received
	 * 
	 * */
	
	public MSCMessage(String _message, String _value, String _from, String _to, int _sendID, int _receiveID){
		this.message	= _message;
		this.value		= _value;
		this.from		= _from;
		this.to			= _to;
		this.sendID		= _sendID;
		this.receiveID	= _receiveID;
	}
	
	/**
	 * Method getMessage() returns a message name
	 * @return message name of message
	 * */
	
	public String getMessage(){		
		return this.message;
	}
	
	/**
	 * Method getValue() returns a message value
	 * @return value value of message
	 * */
	
	public String getValue(){
		return this.value;		
	}
	
	/**
	 * Method getFrom() returns a process id that sent this message
	 * @return from process id that sent message
	 * */
	
	public String getFrom(){
		return this.from;
	}
	
	/**
	 * Method getTo() returns a process id that received this message
	 * @return to process id that received message
	 * */
	
	public String getTo(){
		return this.to;
	}
	
	/**
	 * Method getSendID() returns a line id in which this message was sent
	 * @return sendID line id in which message was sent
	 * */
	
	public int getSendID(){
		return this.sendID;
	}
	
	/**
	 * Method getReceivedID() returns a line id in which this message was received
	 * @return receiveID line id in which message was received
	 * */
	
	public int getReceiveID(){
		return this.receiveID;
	}	
}
