package org.um.feri.spin_msc_generator.message;

import java.util.ArrayList;

import org.um.feri.spin_msc_generator.process.MSCProcess;
import org.um.feri.spin_msc_generator.process.MSCVirtualProcess;

public class MSCMessageRewriter {

	/**
	 * 
	 * MSCMessageRewriter is the main class for rewriting messages. After virtual processes have been created
	 * messages have to be rewritten, because their parent processes do not exist anymore. 
	 * 
	 * */
	
	
	private ArrayList<MSCMessage> rewrittenMessages = new ArrayList();
	
	/**
	 * 
	 * MSCMessageRewritter is a constructor of MSCMessageRewritter class. As a parameter he accepts virtualProcessList
	 * and messageList. Main idea is to go over all messages and check to which process a message belongs. If necessary
	 * the message should be rewritten.
	 * 
	 * For example:
	 * <i>
	 * 	We have three processes P1, P2 and P3.
	 *  If message M1 is sent from P1 to P3 and we create virtual process V1 from P2 and P3, then message should be
	 *  rewritten from P3 to V1.
	 * </i>
	 * 
	 * @param _virtualProcessList
	 * @param _messageList
	 * 
	 * */
		
	public MSCMessageRewriter(ArrayList<MSCVirtualProcess> _virtualProcessList, ArrayList<MSCMessage> _messageList){
		rewriteMessages(_virtualProcessList, _messageList);
	}
	
	/**
	 * 
	 * Method rewriteMessage is the main method for rewriting messages. Method checks all messages and with help
	 * of method rewriteMessage tells if a message should be rewritten. In that case message is rewritten and added
	 * to rewritten message list. 
	 * 
	 * */
	
	private void rewriteMessages(ArrayList<MSCVirtualProcess> _virtualProcessList, ArrayList<MSCMessage> _messageList){
	
		MSCMessage mscMessage;
		MSCMessage newMSCMessage;
		
		for (int i=0; i<_messageList.size(); i++){
			mscMessage = _messageList.get(i);
			newMSCMessage = rewriteMessage(_virtualProcessList, mscMessage);
			
			if (newMSCMessage != null){
				rewrittenMessages.add(newMSCMessage);
			}
		}
	}
	
	
	/**
	 * 
	 * Method rewriteMessage is a method which rewrites a message if a process from which message was sent or delivered
	 * exist in virtual process list. In that case method changes from which process message was sent and which process
	 * received the message.
	 * 
	 * @return newMSCMessage rewritten message
	 * 
	 * */
	
	private MSCMessage rewriteMessage(ArrayList<MSCVirtualProcess> _virtualProcessList, MSCMessage _mscMessage){
		
		MSCMessage 			newMSCMessage = null;
		MSCVirtualProcess	virtualProcess = null;
		
		String		message		= _mscMessage.getMessage();
		String		value		= _mscMessage.getValue();
		String		from		= _mscMessage.getFrom();
		String		to			= _mscMessage.getTo();
		int			sendID		= _mscMessage.getSendID();
		int			receiveID	= _mscMessage.getReceiveID();
		
		
		for (int i=0; i<_virtualProcessList.size(); i++){
			virtualProcess = _virtualProcessList.get(i);
			
			if (from.equals(virtualProcess.getActualID())){
				from = virtualProcess.getNewID();
			}
			if (to.equals(virtualProcess.getActualID())){
				to = virtualProcess.getNewID();
			}
		}
		if (from.equals(to)){
			newMSCMessage = null;
		}
		else {		
			newMSCMessage = new MSCMessage(message, value, from, to, sendID, receiveID);
		}
		
		return newMSCMessage; 
	}
	
	/**
	 * 
	 * Method getRewrittenMessageList returns rewritten messages
	 * @return rewrittenMessages messages that were rewritten on the virtual process list basis
	 * 
	 * */
	

	public ArrayList<MSCMessage> getRewrittenMessageList(){
		return this.rewrittenMessages;
	}
}
