package org.um.feri.spin_msc_generator.msc_parameters;

import java.util.Calendar;

public class MSCDocumentParameters {
	
	/**
	 * 
	 * MSCDocumentParameters class is used for defining basic sizes of MSC document. 
	 * 
	 * */

	private String 			cifMSCDocument		= "";
	private String 			cifMSCPageSize		= "";
	private String 			cifMSCDocumentName	= "";
	private String 			cifToolVersion		= "";
	private String 			cifAuthorName		= "";
	private String 			cifLastModified		= "";
	private String			cifMSC				= "  MSC ";
	private final String	cifEndMSC			= "  ENDMSC;";
	private final String	cifEndMSCDocument	= "ENDMSCDOCUMENT;";
	private final String	mscIS				= " /* MSC IS */ ";
	
	
	/**
	 * 
	 * MSCDocumentParameters 
	 * 
	 * @param _document_width width of current MSC document
	 * @param _document_height height of current MSC document
	 * @param _page_width width of page
	 * @param _page_height height of page
	 * @param _toolName name of tool that generated file
	 * @param _toolVersion version of tool that generated file
	 * @param _authorName name of author that developed tool
	 * @param _toolFormat format used in generated file
	 * 
	 * 
	 * */
	
	public MSCDocumentParameters(int _document_width, int _document_height, int _page_width, int _page_height,
			String _toolName, String _toolVersion, String _authorName, String _toolFormat, String _name){
		
		if (_name == null || _name.equals("")){
			_name = "main";
		}
		
		this.cifMSCDocument		= "/* CIF MSCDOCUMENT (20, 20), (" + _document_width + ", " + _document_height + ") */";
		this.cifMSCPageSize		= "/* CIF MscPageSize (" + _page_width + ", " + _page_height + ") */";
		this.cifMSCDocumentName	= "MSCDOCUMENT " + _name;
		this.cifToolVersion		= "/* CIF " + _toolName + " tool=" + _toolVersion + " format=" + _toolFormat + " cif=yes */";
		this.cifAuthorName		= "/* CIF Author = " + _authorName + " */";
 		// this.cifLastModified	= "/* CIF LastModified "  + getDate() + " */;";
		this.cifLastModified	= "/* CIF LastModified "  +  " */";
		this.cifMSC				= "MSC " + _name + ";";
	}
	
	/**
	 * 
	 * Method returns StringBuffer of document start
	 * 
	 * 
	 * */
	
	public StringBuffer getStartMSCDocument(boolean isMainOfVirtual){
		StringBuffer startDocument = new StringBuffer();
		
			startDocument = startDocument.append(cifMSCDocument + "\n");
		if (!isMainOfVirtual) {
			// startDocument = startDocument.append(cifMSCPageSize + "\n");
			startDocument = startDocument.append(cifMSCDocumentName + ";\n");
		}
	
		if (isMainOfVirtual) {
			startDocument = startDocument.append(cifMSCDocumentName + "\n");
			startDocument = startDocument.append(cifToolVersion + "\n");
			startDocument = startDocument.append(cifAuthorName + "\n");
			startDocument = startDocument.append(cifLastModified + mscIS + ";\n");
		}
		if (!isMainOfVirtual){
			startDocument = startDocument.append(cifMSC + "\n");
		}
		
		return startDocument;		
	}
	
	/**
	 * 
	 * Method returns StringBuffer of document end
	 * 
	 * 
	 * */
	
	public StringBuffer getEndMSCDocument(boolean isMainOfVirtual){
		StringBuffer endDocument = new StringBuffer();
		if (!isMainOfVirtual){
			endDocument = endDocument.append(cifEndMSC + "\n");
		}
		endDocument = endDocument.append(cifEndMSCDocument);
		
		return endDocument;
	}
	
	/**
	 * 
	 * Returns current date...
	 * 
	 * */
	
	private String getDate(){
		Calendar calendar = Calendar.getInstance();
		
		return calendar.getTime().toString();
	}
}