package org.um.feri.spin_msc_generator.msc_parameters;

public class MSCProcessParameters {
	
	/**
	 * 
	 * MSCProcessParameters class is a class for generating MSC compliant output 
	 * 
	 * */
	
	
	int coord_x		= 0;
	int coord_y		= 0;
	
	int head_width	= 0;
	int head_height	= 0;
	
	int proc_width	= 0;
	int proc_height	= 0;
	
	int doc_height	= 0;
	
	int head_middle = 0;
	
	String procID	= "";
	
	public String procName	= "";
	
	private String cifInstance			= "";
	private String instance				= "  INSTANCE";
	private final String endInstance	= "  ENDINSTANCE";
	
	
	/**
	 *
	 * @param _procID ID of process
	 * @param _procName Process name
	 * @param _coord_x offset of coordinate X where process is placed in document
	 * @param _coord_y offset of coordinate Y where process is placed in document
	 * @param _head_middle middle of process head. Variable is used for defining where message should start or where it should end
	 * @param _doc_height 
	 * 
	 * */
	
	public MSCProcessParameters(String _procID, String _procName, int _coord_x, int _coord_y, int _head_middle, int _doc_height){
		
		procID		= _procID;
		procName	= _procName;
		coord_x		= _coord_x;
		coord_y		= _coord_y;
		doc_height	= _doc_height;
		head_middle = _head_middle;
	

		int process_width	= calcProcessWidth(_procName);
		int process_height	= 150; // HEAD
		
		cifInstance = "  /* CIF INSTANCE (" + coord_x  + ", " + coord_y + "), " +
				"(" + process_width + ", " + process_height + "), (0, " + doc_height + ") */";
	}
	
	/**
	 * 
	 * @param procName accepts process name and returns width
	 * @return width
	 *  
	 * */
	
	private int calcProcessWidth(String procName){
		int defaultSize		= 120;		// Default size without text
		int incrementSize 	= 14;		// Char size
		int width = defaultSize + (procName.length()*incrementSize);	// Calculate process width	
				
		return width;
	}
	
	/**
	 * 
	 * Method returns starting description of a process
	 * @return str starting description of a process
	 * 
	 * 
	 * */
	
	public StringBuffer getProcessStart(){
		StringBuffer str = new StringBuffer();
	
		str.append(cifInstance + "\n");
		str.append(instance + " " + procName + ";");
		return str;	
	}
	
	/**
	 * 
	 * Method returns ending description of a process
	 * @return str ending description of a process
	 * 
	 * */
	
	public StringBuffer getProcessEnd(){
		StringBuffer str = new StringBuffer();
		
		str.append(endInstance + ";\n");
		
		return str;
	}
	
	
	/**
	 * 
	 * Method returns head width of a process, which helps generator to determine how wide the process is. 
	 * 
	 * */
	
	public int getHeadWidth(){
		return head_width;
	}
	
	/**
	 * Method return process ID
	 * 
	 * */
	
	public String getProcID(){
		return procID;
	}
	
	/**
	 * 
	 * Method returns head middle - for sending and receiving messages
	 * 
	 * */
	
	public int getHeadMiddle(){
		return head_middle;
	}
}
