package org.um.feri.spin_msc_generator.process;

import java.util.ArrayList;


public class MSCProcessJoiner {

	/**
	 * 
	 * MSCProcessJoiner is the core of using virtual processes. In this class old processes are rewritten to 
	 * new one.
	 * 
	 * */
	
	
	private boolean error = false;
	
	private ArrayList<MSCProcess>			spinProcessesList		= new ArrayList();
	private ArrayList<MSCVirtualProcess>	virtualProcessesList	= new ArrayList();
	
	private ArrayList<MSCVirtualProcess> virtualList = new ArrayList();
	
	private String	userVirtualProcessSelection	= new String();
	
	public MSCVirtualProcess virtualProcess;
	

	/**
	 * 
	 * Constructor MSCProcessJoiner is a constructor for rewriting process from actual process read in from
	 * file to new virtual process
	 * 
	 * */
	
	public MSCProcessJoiner(ArrayList<MSCProcess> _spinProcesses, String _userVirtualProcessSelection){
		this.spinProcessesList = _spinProcesses;
		this.userVirtualProcessSelection = _userVirtualProcessSelection;
		
		virtualProcessesList = createVirtualProcessList(_userVirtualProcessSelection);
	}
	
	/**
	 * 
	 * Function creates list of virtual processes from user arguments
	 * 
	 * <i>
	 * 
	 * User arguments should be entered in this rule 
	 * Example: proxy1,proxy2,proxy3>proxy
	 * if we have multiple rules, than they should be entered like this
	 * Example: proxy1,proxy2,proxy3>proxy webserver1,webserver2,webserver3>webserver
	 * 
	 * </i>
	 * 
	 * 
	 * @param _userArguments User arguments for defining which processes belong to which virtual process.
	 * 
	 * */
	public ArrayList<MSCVirtualProcess> createVirtualProcessList(String _userArguments){
	
		/**
		 * 
		 * Possible input #1: proxy1,proxy2,proxy3>proxy
		 * Possible input #2: proxy1,proxy2,proxy3>proxy webserver1,webserver2,webserver3>webserver
		 * 
		 * Idea of this algorithm is to:
		 * 1. Split various rules to space ' '
		 *   (rule #2: array[0] = proxy1,proxy2,proxy3>proxy array[1]=webserver1,webserver2,webserver3>webserver)
		 * 2. Split already splitted rule to: '>'
		 *   (array[0] = proxy1,proxy2,proxy3 array[1] = proxy)
		 * 3. Split upper rule to comma
		 *   (array[0] = proxy1, array[1] = proxy2, array[2] = proxy3)
		 * 
		 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
		 * 
		 * 4. For every process (proxy1, proxy2, proxy3) we check in processList list if process exists there
		 * 5. If process does not exists we check virtual process in his list.
		 * 6. If virtual process exist we return his ID, which will be the old ID in virtual process list 
		 * 7. If process does not exist we add him to a virtual process list with ID of current process (proxy1, proxy2, proxy3)
		 * 
		 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
		 * 
		 * At this stage we have a new list of virtual processes which can be used afterwards for extraction of processes
		 * 
		 * 
		 * */
		
		String 		argUserArguments = _userArguments;
		String[] 	argSplitSpace = _userArguments.split(" ");
		
		String[] 	argSplitMoreThan;
		
		String[] 	argSplitComma;
		
		String process = "";
		String spinProcessID = "";
		String spinProcessName = "";
		
		String virtualProcess = "";
		String virtualProcessID = "";
		
		MSCVirtualProcess mscVirtualProcess;
		
		for (int i=0; i<argSplitSpace.length; i++){			
			argSplitMoreThan = argSplitSpace[i].split(">");

			argSplitComma = argSplitMoreThan[0].split(",");
			virtualProcess = argSplitMoreThan[1];
			
			for (int j=0; j<argSplitComma.length; j++){
				spinProcessID = argSplitComma[j];
				spinProcessName = getSpinProcessName(spinProcessID);
				
				if (spinProcessName == "-1"){
					// If Spin process with name "process" does not exist throw error.
					System.err.println("Process with ID " + spinProcessID + " is unknown.");
					System.err.println("Check if it is really on the process list.");
					break;
				}
				else{
					// Spin process with name "process" exists in table
					virtualProcessID = getVirtualProcessID(virtualProcess);
					
					if (virtualProcessID.equals("-1")){
						mscVirtualProcess = new MSCVirtualProcess(spinProcessID, spinProcessName, spinProcessID, virtualProcess);
						
						virtualList.add(mscVirtualProcess);
					}
					else {
						mscVirtualProcess = new MSCVirtualProcess(spinProcessID, spinProcessName, virtualProcessID, virtualProcess);
						
						virtualList.add(mscVirtualProcess);
					}
				}
			}
		}
		return virtualList;
	}
	
	/**
	 * 
	 * Method returns name of a process that was detected in input file
	 * 
	 * */
	
	private String getSpinProcessName(String _processID){
		String processName = "-1";
		MSCProcess mscProcess;
		
		for (int i=0; i<spinProcessesList.size(); i++){
			mscProcess = spinProcessesList.get(i);
		
			if (mscProcess.getProcID().equals(_processID)){
				processName = mscProcess.getProcName();
				break;
			}
		}
		return processName;		
	}
	
	/**
	 * 
	 * Method returns ID of a virtual process if he exists in virtual process list. If he doesn't, method returns
	 * -1
	 * 
	 * */
	
	private String getVirtualProcessID(String _virtualName){
		String virtualID = "-1";
		MSCVirtualProcess virtualProcess;
		
		for (int i=0; i<virtualList.size(); i++){
			virtualProcess = virtualList.get(i);
			
			if (virtualProcess.getNewName().equals(_virtualName)){
				virtualID = virtualProcess.getNewID();
				break;
			}
		}
		return virtualID;		
	}
		
	/**
	 * 
	 * Method is used for returning list of virtual processes
	 * 
	 * */
	
	public ArrayList<MSCVirtualProcess> getVirtualProcesses(){
		
		return this.virtualProcessesList;
	}

	
	/**
	 * 
	 * Method getGeneratedProcessList is very important algorithm for creation of a new processes out of
	 * list of actual (from input file) and virtual (from user input) processes.
	 * 
	 * Main idea of this algorithm is:
	 * 1. Algorithm goes through all processes in process list
	 * 2. It checks if a process is added in virtual process list (if he is not, it adds it to processList)
	 * 3. If process is in virtual process list, than we check if virtual process, that this process belongs to exists in process list if it does we overlook it
	 * 4. In the other case we add it to a process list.
	 * 
	 * <i>
	 * 
	 * Example:
	 * 
	 * proxy1, proxy2 belong to proxy virtual process
	 * 
	 * when we go through proxy1 then we have to add new virtual process to process list (only the first time (proxy1))
	 * when we go through proxy2 then we must not add new virtual process, because it already exists in process list
	 * 
	 * the same goes for webserver
	 * 
	 * </i>
	 * 
	 * */

	public ArrayList<MSCProcess> getGeneratedProcessList(ArrayList<MSCProcess> _processList,
			ArrayList<MSCVirtualProcess> _virtualList){
		
		MSCProcess getGeneratedProcessListSingleProcess;
		MSCProcess newProcess;
		
		MSCVirtualProcess getGeneratedProcessListVirtualProcess;
		ArrayList<MSCProcess> getGeneratedProcessListProcessList = new ArrayList();
			
		for (int i=0; i<_processList.size(); i++){
			getGeneratedProcessListSingleProcess = _processList.get(i);
			
			getGeneratedProcessListVirtualProcess = procesExistsInVirtualList(getGeneratedProcessListSingleProcess, _virtualList);
			
			if (getGeneratedProcessListVirtualProcess == null){
				// Because it does not exist in virtual list we add it automaticaly
				getGeneratedProcessListProcessList.add(getGeneratedProcessListSingleProcess);
			}
			else {
				// Because it does exist in virtual list we check if virtual process already exists in process list

				if (processExistsInProcessList(getGeneratedProcessListVirtualProcess, getGeneratedProcessListProcessList)){
				// If it does exist we overlook it
				
				}
				else {
				// If it does not exist we add it to process list
					newProcess = new MSCProcess(getGeneratedProcessListVirtualProcess.getNewID(), getGeneratedProcessListVirtualProcess.getNewName(),
							getGeneratedProcessListSingleProcess.getParentID(), getGeneratedProcessListSingleProcess.getParentName(), getGeneratedProcessListSingleProcess.getProcessLineID());
					getGeneratedProcessListProcessList.add(newProcess);
				}
			}
		}
		return getGeneratedProcessListProcessList;
	}
	
	/**
	 * 
	 * This method checks if process exists in virtual process list.
	 * If process exists method returns his id, otherwise it returns null
	 * 
	 * */
	
	private MSCVirtualProcess procesExistsInVirtualList(MSCProcess _process, ArrayList<MSCVirtualProcess> _virtualList){
		MSCVirtualProcess processExistsInVirutalListVirtualProcess = null;
		
		for (int i=0; i<_virtualList.size(); i++){
			processExistsInVirutalListVirtualProcess = virtualList.get(i);
			
			if (processExistsInVirutalListVirtualProcess.getActualID().equals(_process.getProcID())){
				break;
			}
			else {
				processExistsInVirutalListVirtualProcess = null;
			}
		}
		return processExistsInVirutalListVirtualProcess;
	}
	
	/**
	 * 
	 * This method checks if process exists in process list.
	 * If process exists method returns true, otherwise it returns false
	 * 
	 * */
	
	private boolean processExistsInProcessList(MSCVirtualProcess _virtualProcess, ArrayList<MSCProcess> _processList){
		MSCProcess processExistsInProcessListSingleProcess;
		for (int i=0; i<_processList.size(); i++) {
			processExistsInProcessListSingleProcess = _processList.get(i);
			
			if (processExistsInProcessListSingleProcess.getProcID().equals(_virtualProcess.getNewID())){
				return true;
			}
		}		
		return false;
	}
}
