package org.um.feri.spin_msc_generator.ui;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.um.feri.spin_msc_generator.St2msc;
import org.um.feri.spin_msc_generator.message.MSCMessage;
import org.um.feri.spin_msc_generator.process.MSCProcess;

public class UserInterface {
	
	public UserInterface(){
		
	}
	
	public boolean createVirtualProcesses(){
		String userParameters;

		do {
			System.out.println("Create virtual processes? (0-no, 1-yes)");
			
			userParameters = keyboardRead();
			
			if (userParameters.equals("0")){
				return false;
			}
			if (userParameters.equals("1")){
				return true;	
			}
		}
		while (!userParameters.equals("0") || !userParameters.equals("1"));		

		return false;
	}
	
	public void docParsing(){
		System.out.println("Document parsing: STARTED.");
	}
	public void docParsed(){
		System.out.println("Document parsing: ENDED.");		
	}
	
	public void detectedItems(int numProcesses, int numMessages, int notReceivedMessages){
		System.out.println("Number of detected processes:	" + numProcesses);
		System.out.println("Number of detected messages:");
		System.out.println("	- sent and received:	" + numMessages);
		System.out.println("	- sent not received:	" + notReceivedMessages);
		
	}
	
	public boolean renameMessages(){
		String rename = "";
		
		System.out.println("Rename messages? (0-no, 1-yes)");
		if (keyboardRead().equals("1")){
			return true;
		}
		else {
			return false;
		}
	}
	
	public String whichMessagesToRename(){
		String renamedMessages = "";
		
		do {
			System.out.println("Which messages to rename (Example m1>message1):");
			renamedMessages = keyboardRead();
		}
		while (!isUserMessRegExOK(renamedMessages) && !enteredMessagesOK(renamedMessages));
		return renamedMessages;
	}

	public String addComment(){
		String comment = "";
		
		System.out.println("Add comment? (0-no, 1-yes)");
		if (keyboardRead().equals("1")){
			System.out.println("Please insert comment: ");
			comment = keyboardRead();
		}
		
		return comment;
	}
	
	public boolean showInConsoleOrSave(){
		System.out.println("Show content in console or save to file? (0-Console, 1-File)");
		if (keyboardRead().equals("1")){
			return true;
		}
		return false;	
	}
	
	public String filename(){
		String filename = "";
		
		System.out.println("Enter file name to which you want to save content");
		filename = keyboardRead();
		
		return filename;
	}
	
	public boolean showMessageParameters(){
		System.out.println("Show message parameters? (0-no, 1-yes)");
		if (!keyboardRead().equals("0")){
			return true;
		}
		return false;
	}
	
	public void showUniqueMessages(ArrayList<MSCMessage> messageList){
		
		ArrayList<String> uniqueMessages;
		
		uniqueMessages = getUniqueMessages(messageList);
		
		System.out.println("Unique detected messages: ");
		
		for (int i=0; i<uniqueMessages.size(); i++){
			System.out.print(uniqueMessages.get(i) + " ");
		}
		System.out.println("");
	}
	
	public ArrayList<String> getUniqueMessages(ArrayList<MSCMessage> messageList){
		
		ArrayList<String>		uniqueMessages = new ArrayList();
		MSCMessage				tmpMessage;
		String					uniqueMessageName;
		
		boolean					isOnList = false;
		
		for (int i=0; i<messageList.size(); i++){
			tmpMessage = messageList.get(i);
			
			for (int j=0; j<uniqueMessages.size(); j++){
				uniqueMessageName = uniqueMessages.get(j);
				
				if (uniqueMessageName.equals(tmpMessage.getMessage())){
					isOnList = true;
					break;
				}
				else{
					isOnList = false;
				}
			}
			
			if (isOnList == false){
				uniqueMessages.add(tmpMessage.getMessage());
			}
		}		
		return uniqueMessages;
	}
	
	private boolean enteredMessagesOK(String rnMessageString){
		
		ArrayList<String>	userMessageList = new ArrayList();
		
		String[] argSplitSpace 	= rnMessageString.split(" ");
		String[] argSplitMoreThan;
		
		for (int i=0; i<argSplitSpace.length; i++){
			argSplitMoreThan = argSplitSpace[i].split(">");
			
			if (!messIsOnList(argSplitMoreThan[0], userMessageList)){
				// add to list
				userMessageList.add(argSplitMoreThan[0]);
			}
			else {
				// do not add to list
				System.out.println("~ALREADY ON LIST~");
				return true;				
			}
		}
		return false;
	}
	
	private boolean messIsOnList(String message, ArrayList<String> userMessageList){
		for(int i=0; i<userMessageList.size(); i++){
			if (userMessageList.get(i).equals(message)){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * 
	 * Method prints all Spin processes on screen
	 *
	 * @param processList is a list of all processes
	 *
	 * */
	
	public void showProcesses(ArrayList<MSCProcess> processList){
		MSCProcess mscProcess = null;
		System.out.println("------ Spin processes ------");
		System.out.println("ID	NAME");
		
		for (int i=0; i<processList.size(); i++){
			mscProcess = processList.get(i);
			System.out.println(mscProcess.getProcID() + "	" + mscProcess.getProcName());
		}
		System.out.println("----------------------------");		
	}

	public void showMessages(ArrayList<MSCMessage> messageList){
		MSCMessage mscMessage = null;
		System.out.println("------ Spin messages ------");
		System.out.println("NAME	VALUE	FROM	TO");
		for (int i=0; i<messageList.size(); i++){
			mscMessage = messageList.get(i);
			System.out.println(mscMessage.getMessage() + "	" + mscMessage.getValue() + "	" + mscMessage.getFrom()
					+ "	" + mscMessage.getTo());
		}
		System.out.println("----------------------------");
	}
	
	public String askToJoin(ArrayList<MSCProcess> processList){
		String userParameters = "";
		boolean processParameterOK = false;
		do {
			System.out.println("Which processes to join (Example (ID,ID>newprocess): 1,2>newprocess):");
			userParameters = keyboardRead();
			
			if (isUserProcRegExOK(userParameters)){
				processParameterOK = isProcessParameterOK(userParameters, processList);
			}
			else {
	            System.out.println("Error in user parameters ("+ userParameters + ")");				
			}
		}
		while (!isUserProcRegExOK(userParameters) || !processParameterOK);
		return userParameters;
	}

	
	
	private String keyboardRead(){
		String keyboard = "0"; // default = 0		
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		try {	keyboard = br.readLine(); } 
	    catch (IOException ex) { System.out.println("Error reading from keyboard!"); }
		return keyboard;
	}
	
	/**
	 * 
	 * Method isUserRegExOK returns true, if user parameters for creating virtual processes is valid and false
	 * otherwise. 
	 * 
	 * @param _userParameters
	 * @return boolean of validated user parameters 
	 *
	 * */
	
	private boolean isUserProcRegExOK(String _userParameters){
		boolean userArgBool = false;

		Pattern pattern;
		
		// 1,2,3>cpu1 4,5,6>cpu2
		pattern = Pattern.compile("^(([0-9]+,)*([0-9]+)[>]([a-zA-Z0-9_]+)[ ]*)+$");	
				
        Matcher fit = pattern.matcher(_userParameters);
      
        if (!fit.matches()) {
        	userArgBool = false;
        } 
        else {
        	userArgBool = true;            	
        }
        return userArgBool;
	}
	
	private boolean isUserMessRegExOK(String _userParameters){
		Pattern pattern;
		// 1st param = message id 
		// example: 1>message1 2>message2
		pattern = Pattern.compile("^(([a-zA-Z0-9_]+)[>]([a-zA-Z0-9_]+)[ ]*)+$");	
				
	    Matcher fit = pattern.matcher(_userParameters);
	  
	    if (!fit.matches()) {
	    	return false;
	    } 
	    else {
	    	return true;
	    }
	}
	
	
	private boolean isProcessParameterOK(String userParameters, ArrayList<MSCProcess> processList){
		ArrayList<String> usedProcesses = new ArrayList();
		
		String[]	argSplitSpace;
		String[]	argSplitMoreT;
		String[]	argSplitComma;
		String		argSplitProce;
			
		argSplitSpace = userParameters.split(" ");
		boolean processParameterOK = false;
		
		for (int i=0; i<argSplitSpace.length; i++){
			argSplitMoreT = argSplitSpace[i].split(">");
			argSplitComma = argSplitMoreT[0].split(",");
			argSplitProce  = argSplitMoreT[1];
			
			for (int j=0; j<argSplitComma.length; j++){
				processParameterOK = isProcessInList(argSplitComma[j], processList);
				if (processParameterOK){
					if (processIsUsed(usedProcesses, argSplitComma[j])){
						System.out.println("Process with ID: " + argSplitComma[j] + " is already used");
						return false;
					}
				}
				else {
					System.out.println("Process with ID: " + argSplitComma[j] + " does not exist.");
					return false;
				}
				usedProcesses.add(argSplitComma[j]);
			}
		}
		return true;
	}
	
	private boolean isProcessInList(String actual, ArrayList<MSCProcess> processList){
		MSCProcess process;
		
		for (int i=0; i<processList.size(); i++){
			process = processList.get(i);
			if (process.getProcID().equals(actual)){
				return true;
			}
		}
		return false;
	}
	
	private boolean processIsUsed(ArrayList usedProcesses, String current){
		for (int i=0; i<usedProcesses.size(); i++){
			if (usedProcesses.get(i).equals(current)){
				return true;
			}
		}
		return false;
	}
	
	public void noItemsFound(){
		System.out.println("No items (processes or messages) were found in the input file. Exiting ...");
	}
	
	public void noInputParameters(){
		System.out.println("st2msc: version " + St2msc.st2mscVersion + ", date " + St2msc.st2mscDate);
		System.out.println("No input parameters. See help [-h].");
	}
	
	public void fileSaved(String generatedFile){
		System.out.println("Generated file was saved to: " + generatedFile);
	}	
}
